#!/bin/sh

./sd-example-incoming-call.py &
sleep 2
./sd-example-incoming-file.py &
sleep 2
./sd-example-morning-alarm.py &
sleep 2
./sd-example-much-body-text.py &
sleep 2
./sd-example-using-button-tint-hint.py &
